@@ -21,8 +21,12 @@ import logger from '../lib/logger';
 import { isRequired } from '../lib/utils';
 import pollRbacCache, { getUserRbacFilter } from '../lib/rbacCaching';
 
+export function getPropertiesWithList() {
+  return ['label', 'role', 'port', 'container'];
+}
+
 // Is there a more efficient way?
-function formatResult(results, removePrefix = true) {
+export function formatResult(results, removePrefix = true) {
   const startTime = Date.now();
   const resultList = [];
   while (results.hasNext()) {
@@ -33,6 +37,12 @@ function formatResult(results, removePrefix = true) {
         if (removePrefix) {
           if (record.get(key).properties !== null && record.get(key).properties !== undefined) {
             resultItem = record.get(key).properties;
+            // We need to check each value within the properties list, and then parse them correctly.
+            getPropertiesWithList().forEach((val) => {
+              if (_.get(resultItem, val) && Array.isArray(_.get(resultItem, val))) {
+                resultItem[val] = resultItem[val].join('; ');
+              }
+            });
           } else {
             resultItem[key.substring(key.indexOf('.') + 1)] = record.get(key);
           }
@@ -43,7 +53,11 @@ function formatResult(results, removePrefix = true) {
     });
     resultList.push(resultItem);
   }
+
   logger.perfLog(startTime, 100, 'formatResult()', `Result set size: ${resultList.length}`);
+  if (removePrefix) {
+    return _.uniqBy(resultList, (item) => item._uid);
+  }
   return resultList;
 }
 
@@ -56,8 +70,8 @@ const isNumber = (value) => !Number.isNaN(value * 1);
 //   }
 //   return false;
 // };
-const isDate = (value) => !isNumber(value) && moment(value, 'YYYY-MM-DDTHH:mm:ssZ', true).isValid();
-const isDateFilter = (value) => ['hour', 'day', 'week', 'month', 'year'].indexOf(value) > -1;
+export const isDate = (value) => !isNumber(value) && moment(value, 'YYYY-MM-DDTHH:mm:ssZ', true).isValid();
+export const isDateFilter = (value) => ['hour', 'day', 'week', 'month', 'year'].indexOf(value) > -1;
 // const isVersion = property.toLowerCase().includes('version');
 
 export function getOperator(value) {
@@ -96,9 +110,13 @@ export function getFilterString(filters) {
       const operatorRemoved = value.replace(/^<=|^>=|^!=|^!|^<|^>|^=/, '');
       if (isNumber(operatorRemoved)) { //  || isNumWithChars(operatorRemoved)
         return `n.${filter.property} ${getOperator(value)} ${operatorRemoved}`;
-      } if (isDateFilter(value)) {
+      }
+      if (isDateFilter(value)) {
         return `n.${filter.property} ${getDateFilter(value)}`;
       }
+      if (getPropertiesWithList().includes(filter.property)) {
+        return `('${operatorRemoved}' IN n.${filter.property})`;
+      }
       return `n.${filter.property} ${getOperator(value)} '${operatorRemoved}'`;
     }).join(' OR ')})`);
   });
@@ -106,6 +124,22 @@ export function getFilterString(filters) {
   return resultString;
 }
 
+export function getDataFromValueList(valuesList) {
+  const data = [];
+  valuesList.forEach((value) => {
+    if (Array.isArray(value)) {
+      value.forEach((val) => {
+        if (data.indexOf(val) === -1) {
+          data.push(val);
+        }
+      });
+    } else {
+      data.push(value);
+    }
+  });
+  return data;
+}
+
 function getIPvFamily(redisHost) {
   return new Promise((resolve) => {
     dns.lookup(redisHost, (err, address, family) => {
@@ -283,8 +317,9 @@ export default class RedisGraphConnector {
     const { withClause, whereClause } = await this.createWhereClause([], ['app', 'cluster']);
     const returnClause = "RETURN DISTINCT app._uid, cluster.name='local-cluster' as local, count(DISTINCT cluster._uid) as clusterCount";
     const query = `
-      ${withClause} MATCH ${APPLICATION_MATCH}-->(:Subscription)<--(:Subscription)--(cluster:Cluster) ${whereClause} ${returnClause}
-      UNION ${withClause} MATCH ${APPLICATION_MATCH}-->(:Subscription {cluster: 'local-cluster', localPlacement: 'true', apigroup: 'apps.open-cluster-management.io'})--(cluster:Cluster) ${whereClause} ${returnClause}
+      ${withClause} MATCH ${APPLICATION_MATCH}-[:contains]->(:Subscription)<--(:Subscription)--(cluster:Cluster) ${whereClause} ${returnClause}
+      UNION ${withClause} MATCH ${APPLICATION_MATCH}-[:contains]->(:Subscription {cluster: 'local-cluster', localPlacement: 'true',
+        apigroup: 'apps.open-cluster-management.io'})--(cluster:Cluster) ${whereClause} ${returnClause}
     `;
     return this.executeQuery({ query, removePrefix: false, queryName: 'runAppClustersQuery' });
   }
@@ -294,11 +329,9 @@ export default class RedisGraphConnector {
    */
   async runAppHubSubscriptionsQuery() {
     const { withClause, whereClause } = await this.createWhereClause([], ['app', 'sub']);
-    const matchClause = `MATCH ${APPLICATION_MATCH}-->${SUBSCRIPTION_MATCH}`;
-    const where = whereClause === '' ? 'WHERE' : `${whereClause} AND`;
-    const additionalWhere = 'NOT exists(sub._hostingSubscription)';
-    const returnClause = 'RETURN app._uid, sub._uid, sub.timeWindow, sub.localPlacement, sub.status, sub.channel, sub.name';
-    const query = `${withClause} ${matchClause} ${where} ${additionalWhere} ${returnClause}`;
+    const matchClause = `MATCH ${APPLICATION_MATCH}-[:contains]->${SUBSCRIPTION_MATCH}`;
+    const returnClause = 'RETURN app._uid, sub._uid, sub.timeWindow, sub.localPlacement, sub.status, sub.channel, sub.name, sub.namespace';
+    const query = `${withClause} ${matchClause} ${whereClause} ${returnClause}`;
     return this.executeQuery({ query, removePrefix: false, queryName: 'runAppHubSubscriptionsQuery' });
   }
 
@@ -307,84 +340,18 @@ export default class RedisGraphConnector {
    */
   async runAppHubChannelsQuery() {
     const { withClause, whereClause } = await this.createWhereClause([], ['app', 'sub', 'ch']);
-    const matchClause = `${withClause} MATCH ${APPLICATION_MATCH}-[*1]->${SUBSCRIPTION_MATCH}-[*1]->(ch:Channel)`;
-    const where = whereClause === '' ? 'WHERE' : `${whereClause} AND NOT exists(sub._hostingSubscription)`;
-    const returnClause = 'RETURN app._uid, sub._uid, sub._gitbranch, sub._gitpath, sub._gitcommit, ch._uid, ch.type, ch.pathname';
-    const query = `${matchClause} ${where} ${returnClause}`;
+    const matchClause = `${withClause} MATCH ${APPLICATION_MATCH}-[:contains]->${SUBSCRIPTION_MATCH}-[*1]->(ch:Channel)`;
+    const returnClause = 'RETURN app._uid, sub._uid, sub.name, sub.namespace, sub._gitbranch, sub._gitpath, sub._gitcommit, sub.package, sub.packageFilterVersion, ch._uid, ch.type, ch.pathname';
+    const query = `${matchClause} ${whereClause} ${returnClause}`;
     return this.executeQuery({ query, removePrefix: false, queryName: 'runAppHubChannelsQuery' });
   }
 
-  /*
-   * Get Applications with the pods counter
-   * return the number of pods for this app as a string, grouped by their status
-  */
-  async runAppPodsCountQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'pod']);
-    const query = `${withClause} MATCH ${APPLICATION_MATCH}-->(:Subscription)<--(:Subscription)--(pod:Pod) ${whereClause} RETURN app._uid, pod._uid, pod.status`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runAppPodsCountQuery' });
-  }
-
-  /*
-   * Get Applications with their related remote subscriptions.
-   # Remote subscriptions are those with the '_hostingSubscription' property.
-   */
-  async runAppRemoteSubscriptionsQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'sub']);
-    const matchClause = `MATCH ${APPLICATION_MATCH}-->(:Subscription)<--(sub:Subscription)`;
-    const where = whereClause === '' ? 'WHERE' : `${whereClause} AND`;
-    const additionalWhere = 'exists(sub._hostingSubscription)=true';
-    const query = `${withClause} ${matchClause} ${where} ${additionalWhere} RETURN app._uid, sub._uid, sub.status`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runAppRemoteSubscriptionsQuery' });
-  }
-
-  /*
-   * Get clusters related to any application.
-   */
-  async runGlobalAppClusterCountQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'cluster']);
-    const query = `${withClause} MATCH ${APPLICATION_MATCH}-->(:Subscription)<--(:Subscription)-->(cluster:Cluster) ${whereClause} RETURN DISTINCT cluster._uid`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runGlobalAppClusterCountQuery' });
-  }
-
-  /*
-   * Get channels related to any application.
-   */
-  async runGlobalAppChannelsQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'ch']);
-    const query = `${withClause} MATCH ${APPLICATION_MATCH}<-[]-(ch:Channel) ${whereClause} RETURN DISTINCT ch`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runGlobalAppChannelsQuery' });
-  }
-
-  /*
-   * Get hub subscriptions related to any application.
-   */
-  async runGlobalAppHubSubscriptionsQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'sub']);
-    const matchClause = `MATCH ${APPLICATION_MATCH}-[]->${SUBSCRIPTION_MATCH}`;
-    const where = whereClause === '' ? 'WHERE' : `${whereClause} AND`;
-    const additionalWhere = 'NOT exists(sub._hostingSubscription)';
-    const query = `${withClause} ${matchClause} ${where} ${additionalWhere} RETURN DISTINCT sub`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runGlobalAppHubSubscriptionsQuery' });
-  }
-
-  /*
-   * Get remote subscriptions related to any application.
-   */
-  async runGlobalAppRemoteSubscriptionsQuery() {
-    const { withClause, whereClause } = await this.createWhereClause([], ['app', 'sub']);
-    const matchClause = `MATCH ${APPLICATION_MATCH}-->(:Subscription)<--(sub:Subscription)`;
-    const where = whereClause === '' ? 'WHERE' : `${whereClause} AND`;
-    const additionalWhere = 'exists(sub._hostingSubscription)=true';
-    const query = `${withClause} ${matchClause} ${where} ${additionalWhere} RETURN DISTINCT sub._uid, sub.status`;
-    return this.executeQuery({ query, removePrefix: false, queryName: 'runGlobalAppRemoteSubscriptionsQuery' });
-  }
-
   /*
    * Get Subscriptions.
    */
   async runSubscriptionsQuery() {
     const { withClause, whereClause } = await this.createWhereClause([], ['sub']);
-    const matchClause = `MATCH ${SUBSCRIPTION_MATCH} ${whereClause} AND NOT exists(sub._hostingSubscription)`;
+    const matchClause = `MATCH ${SUBSCRIPTION_MATCH} ${whereClause}`;
     const returnClause = 'RETURN DISTINCT sub._uid, sub.name, sub.namespace, sub.created, sub.selfLink, sub.status, sub.channel, sub.timeWindow, sub.localPlacement';
     const orderClause = 'ORDER BY sub.name, sub.namespace ASC';
     const query = `${withClause} ${matchClause} ${returnClause} ${orderClause}`;
@@ -405,8 +372,9 @@ export default class RedisGraphConnector {
     const { withClause, whereClause } = await this.createWhereClause([], ['sub', 'cluster']);
     const returnClause = "RETURN DISTINCT sub._uid, cluster.name='local-cluster' as local, count(DISTINCT cluster._uid) as clusterCount";
     const query = `
-      ${withClause} MATCH ${SUBSCRIPTION_MATCH}<--(:Subscription)--(cluster:Cluster) ${whereClause} AND NOT exists(sub._hostingSubscription) ${returnClause}
-      UNION ${withClause} MATCH (sub:Subscription {cluster: 'local-cluster', localPlacement: 'true', apigroup: 'apps.open-cluster-management.io'})--(cluster:Cluster) ${whereClause} AND NOT exists(sub._hostingSubscription) ${returnClause}
+      ${withClause} MATCH ${SUBSCRIPTION_MATCH}<--(:Subscription)--(cluster:Cluster) ${whereClause} ${returnClause}
+      UNION ${withClause} MATCH (sub:Subscription {cluster: 'local-cluster', localPlacement: 'true',
+        apigroup: 'apps.open-cluster-management.io'})--(cluster:Cluster) ${whereClause} ${returnClause}
     `;
     return this.executeQuery({ query, removePrefix: false, queryName: 'runSubClustersQuery' });
   }
@@ -423,7 +391,7 @@ export default class RedisGraphConnector {
    */
   async runSubAppsQuery() {
     const { withClause, whereClause } = await this.createWhereClause([], ['sub', 'app']);
-    const matchClause = `MATCH ${SUBSCRIPTION_MATCH}<-[*1]-${APPLICATION_MATCH} ${whereClause} AND NOT exists(sub._hostingSubscription)`;
+    const matchClause = `MATCH ${SUBSCRIPTION_MATCH}<-[:contains]-${APPLICATION_MATCH} ${whereClause}`;
     const returnClause = 'RETURN DISTINCT sub._uid, count(DISTINCT app._uid) as count';
     const query = `${withClause} ${matchClause} ${returnClause}`;
     return this.executeQuery({ query, removePrefix: false, queryName: 'runSubAppsQuery' });
@@ -529,31 +497,8 @@ export default class RedisGraphConnector {
     // logger.info('runSearchQuery()', filters);
     const startTime = Date.now();
     if (this.rbac.length > 0) {
-      // RedisGraph 1.0.15 doesn't support an array as value. To work around this limitation we
-      // encode labels in a single string. As a result we can't use an openCypher query to search
-      // for labels so we need to filter here, which btw is inefficient.
-      const specialFilters = filters.filter((f) => (f.property === 'label' || f.property === 'role'));
-      if (specialFilters) {
-        const { withClause, whereClause } = await this.createWhereClause(filters.filter((f) => f.property !== 'role' && f.property !== 'label'), ['n']);
-        const q = `${withClause} MATCH (n) ${whereClause} RETURN n`;
-        const res = await this.g.query(q);
-        logger.perfLog(startTime, 150, 'SpecialFilterSearchQuery');
-        return formatResult(res).filter((item) => {
-          if (item.role && specialFilters.findIndex((filter) => filter.property === 'role') >= 0) {
-            const roleIdx = specialFilters.findIndex((filter) => filter.property === 'role');
-            return specialFilters[roleIdx].values.find((value) => {
-              const values = value.replace(' ', '').split(',');
-              return values.every((v) => item.role.includes(v));
-            });
-          }
-          if (item.label && specialFilters.findIndex((filter) => filter.property === 'label') >= 0) {
-            const labelIdx = specialFilters.findIndex((filter) => filter.property === 'label');
-            return item.label && specialFilters[labelIdx].values.find((value) => item.label.indexOf(value) > -1);
-          }
-          return item;
-        });
-      }
-
+      // RedisGraph 2.0 does support an array as value. Therefore, we don't need to
+      // encode labels in a single string
       let limitClause = '';
       if (limit > 0) {
         limitClause = querySkipIdx > -1
@@ -573,13 +518,8 @@ export default class RedisGraphConnector {
     // logger.info('runSearchQueryCountOnly()', filters);
 
     if (this.rbac.length > 0) {
-      // RedisGraph 1.0.15 doesn't support an array as value. To work around this limitation we
-      // encode labels in a single string. As a result we can't use an openCypher query to search
-      // for labels so we need to filter here, which btw is inefficient.
-      const labelFilter = filters.find((f) => (f.property === 'label' || f.property === 'role'));
-      if (labelFilter) {
-        return this.runSearchQuery(filters, -1, -1).then((r) => r.length);
-      }
+      // RedisGraph 2.0 does support an array as value. Therefore, we don't need to
+      // encode labels in a single string
       const { withClause, whereClause } = await this.createWhereClause(filters, ['n']);
       const startTime = Date.now();
       const result = await this.g.query(`${withClause} MATCH (n) ${whereClause} RETURN count(n)`);
@@ -626,8 +566,7 @@ export default class RedisGraphConnector {
         ? ''
         : `LIMIT ${limit}`;
       let f = filters.length > 0 ? filters : [];
-      // Workaround for node resource queries - this will need to be removed when redis 2.0 features are used
-      f = filters.filter((filter) => filter.property !== 'role' && filter.property !== 'label');
+      f = filters.filter((filter) => filter.property);
       const { withClause, whereClause } = await this.createWhereClause(f, ['n']);
       const result = await this.g.query(`${withClause} MATCH (n) ${whereClause} RETURN DISTINCT n.${property} ORDER BY n.${property} ASC ${limitClause}`);
       logger.perfLog(startTime, 500, 'getAllValues()');
@@ -637,32 +576,8 @@ export default class RedisGraphConnector {
         }
       });
 
-      // RedisGraph 1.0.15 doesn't support an array as value. To work around this limitation we
-      // encode labels in a single string. Here we need to decode the string to get all labels.
-      if (property === 'label') {
-        const labels = [];
-        valuesList.forEach((value) => {
-          value.split('; ').forEach((label) => {
-            // We don't want duplicates, so we check if it already exists.
-            if (labels.indexOf(label) === -1) {
-              labels.push(label);
-            }
-          });
-        });
-        return labels;
-      }
-      // Same workaround as above, but for roles.
-      if (property === 'role') {
-        const roles = [];
-        valuesList.forEach((value) => {
-          value.split(', ').forEach((role) => {
-            // We don't want duplicates, so we check if it already exists.
-            if (roles.indexOf(role) === -1) {
-              roles.push(role);
-            }
-          });
-        });
-        return roles;
+      if ((getPropertiesWithList()).includes(property)) {
+        return getDataFromValueList(valuesList);
       }
 
       if (isDate(valuesList[0])) {
